// Main Home project screen
//written by Webdust21 (2024-07-08)
#include <Preferences.h>
#include "screen_base.hpp"      		// because we're deriving from this class
#include "ESP_Panel_Board_Custom.h"		// for screen size definitions

class HomeScreen_Main : public HomeScreen // sub-class
{
private:
	lv_obj_t *solarMeter, *solarArcs[3];

public:
    HomeScreen_Main(lv_obj_t* lvContainer) {
        // Class constructor -> create all necessary LVGL objects
        this->lvContainerObj = lvContainer;     // save our container...

		this->solarMeter = lv_scale_create(this->lvContainerObj);
		lv_obj_set_size(this->solarMeter, 250, 250);
		lv_obj_align(this->solarMeter, LV_ALIGN_TOP_LEFT, LV_PCT(4), LV_PCT(45));
		lv_scale_set_mode(this->solarMeter, LV_SCALE_MODE_ROUND_INNER);
		lv_obj_remove_flag(this->solarMeter, LV_OBJ_FLAG_SCROLLABLE);  // Not scrollable.  (Setting the LV_ARCs to >100% causes this to be "scrollable")
		lv_obj_set_style_pad_all(this->solarMeter, 50, LV_PART_MAIN);
		lv_obj_set_style_radius(this->solarMeter, LV_RADIUS_CIRCLE, 0);
		lv_obj_set_style_clip_corner(this->solarMeter, true, 0);
		//DEBUG
//		lv_obj_set_style_border_color(this->solarMeter, lv_color_white(), LV_PART_MAIN);		// white border
//		lv_obj_set_style_border_width(this->solarMeter, 1, LV_PART_MAIN);						// set border width

		lv_scale_set_label_show(this->solarMeter, true);
		lv_scale_set_total_tick_count(this->solarMeter, 31);
		lv_scale_set_major_tick_every(this->solarMeter, 5);
		lv_scale_set_post_draw(this->solarMeter, true);

		lv_obj_set_style_length(this->solarMeter, 5, LV_PART_ITEMS);
		lv_obj_set_style_length(this->solarMeter, 10, LV_PART_INDICATOR);
		lv_scale_set_range(this->solarMeter, 0, 4000);

		lv_scale_set_angle_range(this->solarMeter, 270);
		lv_scale_set_rotation(this->solarMeter, 135);

		// Manually add "arc" angles for the solar charger outputs
		for (int idx = 0; idx < 3; idx++) {
			this->solarArcs[idx] = lv_arc_create(this->solarMeter);
			lv_arc_set_rotation(this->solarArcs[idx], 135);
			lv_arc_set_bg_angles(this->solarArcs[idx], 0, 270);
			lv_arc_set_range(this->solarArcs[idx], 0, 4000);
			lv_obj_set_style_arc_rounded(this->solarArcs[idx], 0, LV_PART_MAIN);		// background
			lv_obj_set_style_arc_rounded(this->solarArcs[idx], 0, LV_PART_INDICATOR);	// actual indicator
			lv_obj_set_style_arc_color(this->solarArcs[idx], lv_color_make((idx == 2) ? 255 : 0, (idx == 1) ? 255 : 0, (idx == 0) ? 255 : 0), LV_PART_INDICATOR);
			lv_obj_remove_style(this->solarArcs[idx], nullptr, LV_PART_KNOB);   	// Hide the knob
			lv_obj_remove_flag(this->solarArcs[idx], LV_OBJ_FLAG_CLICKABLE);  // Not adjustable by click
			lv_obj_set_size(this->solarArcs[idx], LV_PCT(120 + (idx * 20)), LV_PCT(120 + (idx * 20)));
			lv_obj_center(this->solarArcs[idx]);
		}

	};


/*		lv_arc_t * arc = (lv_arc_t *)this->solarArcs[0];
		log_i("Arc Range: %d to %d", arc->min_value, arc->max_value);  -> where max = 4000.
		lv_arc_set_value(this->solarArcs[0], 8193);		-> crash with debug output of:
			[Warn] lv_draw_buf_create: No memory: 185x43, cf: 16, stride: 740, 31820Byte, 
			[Warn] lv_draw_layer_alloc_buf: Allocating layer buffer failed. Try later
			[Warn] lv_draw_buf_create: No memory: 185x43, cf: 16, stride: 740, 31820Byte, 
			[Warn] lv_draw_layer_alloc_buf: Allocating layer buffer failed. Try later
			--ad infinitum--

			-> increase memory in lv_conf.h to 128KB, problem solved.
*/

	virtual bool Update() override {
        // Handle touch events and redraws here.  Called 10x/sec
		int MpptWattAcc = 0, MpptVal;
		
		// Set the arcs to high values.  Note that if the arcs are set to low values, 
		//the out-of-memory incident does not occur.
		lv_arc_set_value(this->solarArcs[0], 4096);
		lv_arc_set_value(this->solarArcs[1], 5242);
		lv_arc_set_value(this->solarArcs[2], 3950);
		
		return true;
	};

	 // Class destructor (when "delete" is called)
    virtual ~HomeScreen_Main() {
		
	}

};